/* eslint-disable no-nested-ternary */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import raf from 'raf';
import classNames from 'classnames/bind';
import throttle from 'lodash.throttle';
import { navsType } from '../../../utils/propTypes';
import gaDataset from '../../../utils/gaDataset';
import AnueLink from '../../../utils/anueLink';
import { screenSizeXl, screenSizeLg, screenSizeMd } from '../../../styles/_vars.scss';
import AnueSearch from '../../AnueSearch/AnueSearch';
import AnueSubMenu from './AnueSubMenu';
import navigatorConfig from '../config/navs.json';
import styles from '../AnueNavigator.scss';
import { FIXED_HEADER_NONE, DISTANCE_OVER_HEADER, FIXED_HEADER_FULL } from '../constants';

const cx = classNames.bind(styles);

function checkIsFixedHeader(fixedHeaderType) {
  return fixedHeaderType !== FIXED_HEADER_NONE;
}

function convertPxToNumber(size) {
  return parseInt(size.replace('px', ''), 10);
}

export default class AnueNavigator extends Component {
  static propTypes = {
    channelName: PropTypes.string,
    logoUrl: PropTypes.string,
    // TODO: 之前舊版的 header 是宣告一個 Link 的屬性，用來解決編譯為靜態頁面後的連結失效問題
    // channelComponent: PropTypes.oneOfType(PropTypes.element, PropTypes.func),
    navs: navsType,
    searchKeyword: PropTypes.string,
    dataPrefix: PropTypes.arrayOf(PropTypes.string),
    enableSearchBar: PropTypes.bool,
    fixedHeaderType: PropTypes.oneOf([FIXED_HEADER_NONE, FIXED_HEADER_FULL]),
    accountComponent: PropTypes.element,
    switchVersionComponent: PropTypes.element,
    apiEnv: PropTypes.string.isRequired,
    hideCountsWhenSearchExpand: PropTypes.arrayOf(PropTypes.number),
  };

  static defaultProps = {
    channelName: '首頁',
    logoUrl: '',
    navs: null,
    searchKeyword: '',
    // channelComponent: null,
    dataPrefix: ['data-global-ga'],
    enableSearchBar: true,
    accountComponent: null,
    switchVersionComponent: null,
    fixedHeaderType: FIXED_HEADER_FULL,
    hideCountsWhenSearchExpand: [6, 3],
  };

  constructor(props) {
    super(props);

    this.state = {
      fixed: false,
      currentChannelIndex: -1,
      isExpandSearchInput: false,
      isExpandAnimationStart: false,
    };

    // 使用比 React 16.2 還舊的版本，你可以使用這個不同的方式並把 ref 當作另個有名字的 prop 來傳進去
    // https://zh-hant.reactjs.org/docs/refs-and-the-dom.html#callback-refs
    this.setHeaderRef = element => {
      this.headerRef = { current: element };
    };

    this.fixedState = this.state.fixed;
    this.nextStateAnimationFrameId = null;
    this.tabletVerticalMode = null;
    this.tabletLandscapeMode = null;
    this.enableSearchInput = false;

    this.anueUrl = new AnueLink(this.props.apiEnv);
    this.screenWidth = typeof document !== 'undefined' ? document.documentElement.offsetWidth : 1200;

    this.channels = this.props.navs != null ? this.props.navs : navigatorConfig;
    this.channelRef = new Array(this.channels.length);
  }

  componentDidMount() {
    const { hideCountsWhenSearchExpand, fixedHeaderType } = this.props;

    // 當畫面介在 768 ~ 1200 之間，搜尋框是縮小的
    const tabletLandscapeSize = convertPxToNumber(screenSizeMd);
    const tabletVerticalSize = convertPxToNumber(screenSizeLg);
    const desktopMinimalVerticalSize = convertPxToNumber(screenSizeXl);

    const hideCountInDesktop = hideCountsWhenSearchExpand[0] || 6;
    const hideCountInTablet = hideCountsWhenSearchExpand[1] || 4;

    this.hideCounts = this.screenWidth <= tabletVerticalSize ? hideCountInDesktop : hideCountInTablet;
    this.isFixedHeader = checkIsFixedHeader(fixedHeaderType);

    this.tabletVerticalMode = this.screenWidth > tabletLandscapeSize && this.screenWidth <= desktopMinimalVerticalSize;
    this.tabletLandscapeMode = this.screenWidth <= tabletLandscapeSize;

    if (this.isFixedHeader) {
      if (typeof window !== 'undefined') {
        window.addEventListener('scroll', this.scrollHandler);
        // The clicked event of AnueSearch is set to stopPropagation.
        // So click the anywhere outside of AnueSearch will hide the AnueRealtimeResult.
        window.addEventListener('click', this.restoreSearchInputHandler);
      }
    }

    window.addEventListener('resize', throttle(this.screenResizeHandler, 200));
  }

  componentWillUnmount() {
    if (this.isFixedHeader) {
      if (typeof window !== 'undefined') {
        window.removeEventListener('scroll', this.scrollHandler);
        window.removeEventListener('click', this.restoreSearchInputHandler);
      }
    }

    window.removeEventListener('resize', throttle(this.screenResizeHandler, 200));
  }

  /**
   * 點擊搜尋框，啟動搜尋框展開/縮回效果
   */
  onEnableSearchInput = enabled => {
    const { enableSearchBar } = this.props;

    if (!enableSearchBar) return;

    if (this.tabletVerticalMode || this.tabletLandscapeMode) {
      this.enableSearchInput = enabled;
      this.setState({ isExpandAnimationStart: true });
    }
  };

  setNextState = state => {
    if (this.nextStateAnimationFrameId) {
      raf.cancel(this.nextStateAnimationFrameId);
    }

    this.nextStateAnimationFrameId = raf(() => {
      this.nextStateAnimationFrameId = null;
      this.fixedState = state;
      this.setState(prevState => ({ ...prevState, fixed: !prevState.fixed }));
    });
  };

  screenResizeHandler = () => {
    if (document && document.documentElement) {
      this.screenWidth = document.documentElement.offsetWidth;

      const tabletLandscapeSize = convertPxToNumber(screenSizeMd);
      const desktopMinimalVerticalSize = convertPxToNumber(screenSizeXl);

      this.tabletVerticalMode =
        this.screenWidth > tabletLandscapeSize && this.screenWidth <= desktopMinimalVerticalSize;
      this.tabletLandscapeMode = this.screenWidth <= tabletLandscapeSize;
    }
  };

  /**
   * 監聽畫面捲動，為了處理 Heder 固定於最上端的排列
   */
  scrollHandler = () => {
    const { fixedHeaderType } = this.props;

    const scrollTop = document ? document.documentElement.scrollTop : 0;
    const scrollY = 'scrollY' in window ? window.scrollY : scrollTop;
    const isFixed = checkIsFixedHeader(fixedHeaderType);

    if (isFixed) {
      if (scrollY > DISTANCE_OVER_HEADER && !this.fixedState) {
        this.setNextState(true);
      } else if (scrollY <= DISTANCE_OVER_HEADER && this.fixedState) {
        this.setNextState(false);
      }
    }
  };

  /**
   * 點擊搜尋框的放大鏡圖示
   */
  searchIconClickHandler = () => {
    const { enableSearchBar } = this.props;

    if (!enableSearchBar) return;

    if (this.tabletVerticalMode || this.tabletLandscapeMode) {
      this.enableSearchInput = !this.enableSearchInput;
    }
  };

  searchInputFocusHandler = () => {
    if (!this.enableSearchInput) {
      this.onEnableSearchInput(true);
    }
  };

  restoreSearchInputHandler = () => {
    if (this.enableSearchInput) {
      this.onEnableSearchInput(false);
    }
  };

  /**
   * 搜尋框展開動畫效果結束
   */
  animationEndHandler = () => {
    this.setState(prevState => ({
      ...prevState,
      isExpandSearchInput: !prevState.isExpandSearchInput,
      isExpandAnimationStart: false,
    }));
  };

  channelMouseEnterHandler = index => {
    this.setState({ currentChannelIndex: index });
  };

  renderNavigatorChannels = () =>
    this.channels.map((channel, index) => {
      if (!channel.enable) return null;

      const { channelName, dataPrefix, apiEnv } = this.props;
      const { isExpandSearchInput, currentChannelIndex } = this.state;

      const haveSubMenus = channel.subItems.length > 0;
      const isActive = index === currentChannelIndex;
      const needHide = this.enableSearchInput && index < this.hideCounts;
      const isCurrentNav = channelName === channel.title;
      const link = channel.url !== '' ? this.anueUrl.generateLink(channel.url) : '';

      const setChannelRef = element => {
        if (element !== null) {
          this.channelRef[index] = { current: element };
        }
      };

      return (
        <li
          key={channel.name}
          ref={setChannelRef}
          className={cx({ hide: needHide, active: isActive, current: isCurrentNav })}
          onMouseEnter={() => this.channelMouseEnterHandler(index)}
          onMouseLeave={() => this.channelMouseEnterHandler(-1)}
        >
          <div>
            {needHide ? (
              <span>{channel.title}</span>
            ) : link !== '' ? (
              <a
                href={link}
                target={channel.blank ? '_blank' : '_self'}
                rel="noopener noreferrer"
                {...gaDataset({ dataPrefix, category: 'Nav', action: 'click', label: channel.title })}
              >
                <span className={cx({ new: channel.isNew })}>{channel.title}</span>
              </a>
            ) : (
              <span className={cx('no-link', { new: channel.isNew })}>{channel.title}</span>
            )}
          </div>
          {this.headerRef &&
            haveSubMenus && (
              <AnueSubMenu
                key={`${channel.name}_sub_menus`}
                parentName={channel.title}
                headerRef={this.headerRef}
                parentRef={this.channelRef[index]}
                data={channel.subItems || []}
                expandSearchBar={isExpandSearchInput}
                apiEnv={apiEnv}
              />
            )}
        </li>
      );
    });

  render() {
    const {
      searchKeyword,
      accountComponent,
      switchVersionComponent,
      apiEnv,
      dataPrefix,
      logoUrl,
      enableSearchBar,
    } = this.props;
    const { isExpandAnimationStart, isExpandSearchInput, fixed } = this.state;

    let searchInputWrapStyle = null;
    let searchInputStyle = null;
    let channelUnfoldStyle = null;

    // 1024 尺寸時
    if (this.tabletVerticalMode || this.tabletLandscapeMode) {
      const searchInputWidth = this.tabletLandscapeMode ? 270 : 314;

      searchInputWrapStyle = this.enableSearchInput
        ? {
            position: 'absolute',
            right: 0,
            width: `${searchInputWidth}px`,
            maxWidth: '314px',
            transition: 'width 0.2s',
          }
        : null;

      // 利用 text-indent 來達成隱藏 placeholder 的效果
      searchInputStyle = this.enableSearchInput ? { textIndent: '2px' } : null;
      channelUnfoldStyle = this.enableSearchInput
        ? cx('moveLeft', 'expansion')
        : isExpandSearchInput
          ? cx('moveRight')
          : null;
    }

    const searchIconStyle = {
      top: '5px',
      width: '24px',
      height: '24px',
    };
    const indexUrl = logoUrl !== '' ? logoUrl : this.anueUrl.wwwChannelLink;

    return (
      <div className={cx('react-fragment')}>
        {fixed && <div className={cx('anue-navigator-wrapper')} />}
        <nav className={cx('anue-navigator-wrapper', { sticky: fixed })}>
          <header ref={this.setHeaderRef} className={cx('main-header')}>
            <div className={cx('channel-bar')}>
              <a
                href={indexUrl}
                className={cx('logo')}
                {...gaDataset({ dataPrefix, category: 'Logo', action: 'click', label: 'home' })}
              />
              <div className={cx('channel-items', { mask: isExpandAnimationStart })}>
                <ul className={channelUnfoldStyle} onAnimationEnd={this.animationEndHandler}>
                  {this.renderNavigatorChannels()}
                </ul>
              </div>
            </div>
            {enableSearchBar && (
              <div className={cx('search-bar')}>
                <AnueSearch
                  apiEnv={apiEnv}
                  enableNewStyle
                  placeholder="搜尋新聞、行情代碼或名稱"
                  shouldAlwaysDisplayInput
                  onIconClick={this.searchIconClickHandler}
                  onInputFocus={this.searchInputFocusHandler}
                  customWrapperStyles={searchInputWrapStyle}
                  customInputStyles={searchInputStyle}
                  customIconStyles={searchIconStyle}
                  defaultValue={searchKeyword}
                  theme="desktop"
                />
              </div>
            )}
            <div className={cx('info-bar')}>
              {accountComponent && accountComponent}
              {switchVersionComponent && switchVersionComponent}
            </div>
          </header>
        </nav>
      </div>
    );
  }
}
