/* eslint-disable react/no-array-index-key */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import throttle from 'lodash.throttle';
import gaDataset from '../../../utils/gaDataset';
import AnueLink from '../../../utils/anueLink';
import ArrowLeft from '../../../assets/icon-arrow-left.svg';
import ArrowRight from '../../../assets/icon-arrow-right.svg';
import { DIRECTION } from '../constants';
import styles from '../AnueNavigator.scss';

const cx = classNames.bind(styles);

export default class AnueSubMenu extends Component {
  static propTypes = {
    data: PropTypes.arrayOf(
      PropTypes.shape({
        title: PropTypes.string.isRequired,
        url: PropTypes.string.isRequired,
      })
    ).isRequired,
    parentName: PropTypes.string,
    headerRef: PropTypes.element.isRequired,
    parentRef: PropTypes.element.isRequired,
    apiEnv: PropTypes.string.isRequired,
    dataPrefix: PropTypes.arrayOf(PropTypes.string),
    expandSearchBar: PropTypes.bool,
  };

  static defaultProps = {
    parentName: '',
    dataPrefix: ['data-global-ga'],
    expandSearchBar: false,
  };

  constructor(props) {
    super(props);

    this.setRef = element => {
      this.ref = { current: element };
    };
    this.setMenusRef = element => {
      this.menusRef = { current: element };
    };

    this.offset = null;

    this.state = {
      needShowLeftArrow: false,
      needShowRightArrow: false,
    };

    this.anueUrl = new AnueLink(this.props.apiEnv);
  }

  /**
   * 畫面 Resize 需重新計算次選單的座標位置
   */
  componentDidMount() {
    window.addEventListener('resize', throttle(this.handlePosition, 200));
    this.handlePosition();
  }

  /**
   * 配合主導覽列上的搜尋框展開，需要重新計算次選單的座標位置
   */
  componentDidUpdate(prevProps) {
    if (prevProps.expandSearchBar !== this.props.expandSearchBar) {
      this.handlePosition(false);
    }
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.handlePosition);
  }

  /**
   * React events are actually Synthetic Events, not Native Events.
   * Event delegation: React doesn't actually attach event handlers to the nodes themselves.
   * When React starts up, it starts listening for all events at the top level using a single event listener.
   * When a component is mounted or unmounted, the event handlers are simply added or removed from an internal mapping.
   * When an event occurs, React knows how to dispatch it using this mapping.
   * When there are no event handlers left in the mapping,
   * React's event handlers are simple no-ops.
   *
   * Use `Event.stopImmediatePropagation`
   *
   * https://stackoverflow.com/questions/36316846/react-onclick-and-preventdefault-link-refresh-redirect
   */
  onMoveSubMenuLeft = e => {
    e.stopPropagation();
    e.nativeEvent.stopImmediatePropagation();

    this.moveSubMenuHandler(DIRECTION.LEFT);
  };

  onMoveSubMenuRight = e => {
    e.stopPropagation();
    e.nativeEvent.stopImmediatePropagation();

    this.moveSubMenuHandler(DIRECTION.RIGHT);
  };

  /**
   * 對應設計稿，次選單的呈現主要有三種狀況
   * 1. 如果滑出的選單置中情況下，會超出 header 的邊界 (1200, 1024)，則直接對齊邊界
   * 2. 如果滑出的選單置中情況下，不會超出 header 的邊界，則置中對齊
   * 3. 如果選單本身的寬度就已經超過 header (1200, 1024)時，則超出邊界全版顯示
   * 設計稿: https://zpl.io/aMy3zN3
   */
  handlePosition = (needRestore = true) => {
    const { headerRef, parentRef } = this.props;

    if (!this.ref.current) return;

    if (needRestore) {
      this.ref.current.style.width = 'auto';
      this.setState({ needShowLeftArrow: false, needShowRightArrow: false });
    }

    const paddingValue = 16;
    // 上層 Header，用來判斷內容區域的邊界值和寬度
    const headerBoundary = headerRef.current.getBoundingClientRect();
    // 上層 Channel 物件，用來計算次選單顯示座標定位
    const parentBoundary = parentRef && parentRef.current ? parentRef.current.getBoundingClientRect() : { x: 0 };
    // 次選單物件，用來取得目前選單寬度和預設顯示座標
    const subMenuBoundary = this.ref.current.getBoundingClientRect();

    // 次選單寬度超出 header 的邊界，全版顯示
    if (subMenuBoundary.width >= headerBoundary.width) {
      const isOverScreenWidth = subMenuBoundary.width - document.documentElement.offsetWidth >= 0;

      // 顯示滿版情況下的左右箭頭
      if (isOverScreenWidth) {
        this.setState(prevState => ({ ...prevState, needShowRightArrow: true }));

        // 次選單需要滿版顯示
        this.offset = {
          left: `-${parentBoundary.x}px`,
          width: `${document.documentElement.offsetWidth}px`,
        };
      } else {
        // 當次選單寬度超出 header 寬度時，但沒有超出螢幕寬度時，採置中對齊 - [QA-1647]
        const subMenuCenterPosition =
          Math.abs(parentBoundary.x - headerBoundary.x) +
          Math.round((subMenuBoundary.width - headerBoundary.width) / 2);

        this.offset = { left: `-${subMenuCenterPosition}px` };
      }
    } else {
      const subMenuAndHeaderPositionDiff = Math.abs(parentBoundary.x - headerBoundary.x);

      if (subMenuAndHeaderPositionDiff > 0) {
        const subMenuPosition = Math.round(subMenuBoundary.width / 2) - paddingValue * 2;

        this.offset =
          subMenuAndHeaderPositionDiff < subMenuPosition
            ? { left: `-${subMenuAndHeaderPositionDiff}px` }
            : { left: `-${subMenuPosition}px` };
      }
    }
  };

  /**
   * 次選單項目超出畫面時的左右移動處理
   */
  moveSubMenuHandler = direction => {
    let currentDistance = this.menusRef.current.scrollLeft;

    const needMoveCount = Math.ceil(this.menusRef.current.scrollWidth / this.menusRef.current.offsetWidth) - 1;
    const moveDistance = (this.menusRef.current.scrollWidth - this.menusRef.current.offsetWidth) / needMoveCount;

    currentDistance = direction === DIRECTION.RIGHT ? currentDistance + moveDistance : currentDistance - moveDistance;
    this.menusRef.current.scrollLeft = currentDistance;

    if (currentDistance > 0) {
      this.setState(prevState => ({ ...prevState, needShowLeftArrow: true }));
    } else if (currentDistance <= 0) {
      this.setState(prevState => ({ ...prevState, needShowLeftArrow: false }));
    }

    const currentScrollWidth = this.menusRef.current.offsetWidth + currentDistance;

    if (currentScrollWidth < this.menusRef.current.scrollWidth) {
      this.setState(prevState => ({ ...prevState, needShowRightArrow: true }));
    } else if (currentScrollWidth >= this.menusRef.current.scrollWidth) {
      this.setState(prevState => ({ ...prevState, needShowRightArrow: false }));
    }
  };

  /**
   * 展開第二層選單項目
   * 淺灰色連結
   */
  renderSubSecondMenuItems = (parent, first, second) => {
    const { parentRef, dataPrefix, parentName } = this.props;
    // parentName 是配合靜態頁面生成使用，靜態生成時無法動態取得 parentRef 的值，因為是在 Server 端生成，不是在 Browser 上建立
    const rootTitle = parentName || (parentRef && parentRef.current && parentRef.current.children[0].innerText);

    const { subItems: items } = parent;
    const secondMenus = items.filter(item => item.enable).map((item, index) => (
      <li key={`sub_main_menu_${first}_${second}_${index}`}>
        <a
          className={cx({ highlight: item.highlight })}
          href={this.anueUrl.generateLink(item.url)}
          target={item.blank ? '_blank' : '_self'}
          rel="noopener noreferrer"
          {...gaDataset({
            dataPrefix,
            category: 'Nav',
            action: 'click',
            label: `${rootTitle}_${parent.title}_${item.title}`,
          })}
        >
          {item.title}
        </a>
      </li>
    ));

    return <ul className={cx('channel-item-sub-menu-second')}>{secondMenus}</ul>;
  };

  /**
   * 顯示每一欄的第一層選單項目
   * 粗體連結
   */
  renderSubMainMenuItems = (items, first) => {
    if (items.length <= 0) return [];

    const { parentRef, dataPrefix, parentName } = this.props;
    // parentName 是配合靜態頁面生成使用，靜態生成時無法動態取得 parentRef 的值，因為是在 Server 端生成，不是在 Browser 上建立
    const rootTitle = parentName || (parentRef && parentRef.current && parentRef.current.children[0].innerText);

    return items.filter(item => item.enable).map((item, index) => {
      const haveSecondMenu = item.subItems && item.subItems.length > 0;

      return (
        <li key={`sub_main_menu_${first}_${index}`}>
          {item.url !== '' ? (
            <a
              className={cx({ highlight: item.highlight })}
              href={this.anueUrl.generateLink(item.url)}
              target={item.blank ? '_blank' : '_self'}
              rel="noopener noreferrer"
              {...gaDataset({ dataPrefix, category: 'Nav', action: 'click', label: `${rootTitle}_${item.title}` })}
            >
              {item.title}
            </a>
          ) : (
            item.title
          )}
          {haveSecondMenu && this.renderSubSecondMenuItems(item, first, index)}
        </li>
      );
    });
  };

  /**
   * 依照第一層的 UL 個數，建立次選單中的欄
   */
  renderSubMainMenuColumn = () => {
    const { data } = this.props;

    if (data.length <= 0) return [];

    return data.map((menu, index) => {
      return <ul key={`sub_main_menu_${index}`}>{this.renderSubMainMenuItems(menu, index)}</ul>;
    });
  };

  render() {
    const { needShowLeftArrow, needShowRightArrow } = this.state;

    return (
      <div ref={this.setRef} className={cx('channel-item-sub-menu')} style={this.offset}>
        {needShowLeftArrow && (
          <div className={cx('channel-item-sub-menu-arrow', 'left')}>
            <img src={ArrowLeft} alt="Previous" onClick={this.onMoveSubMenuLeft} />
          </div>
        )}
        <div ref={this.setMenusRef} className={cx('channel-item-sub-menus')}>
          {this.renderSubMainMenuColumn()}
        </div>
        {needShowRightArrow && (
          <div className={cx('channel-item-sub-menu-arrow', 'right')}>
            <img src={ArrowRight} alt="Next" onClick={this.onMoveSubMenuRight} />
          </div>
        )}
      </div>
    );
  }
}
