import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import styles from './Crypto.scss';
import gaDataset from '../../utils/gaDataset';
import { COMPANIGN_URL } from './Crypto';

const cx = classNames.bind(styles);

export const CARD_TYPE = {
  LEFT: 'left',
  TOP: 'top',
  BOTTOM: 'bottom',
};

const renderCompanyLogo = logo => {
  return <img src={logo} alt="" />;
};

const renderSymbolIcon = symbol => {
  return <img src={`${COMPANIGN_URL}/assets/crypto-${symbol}.svg`} alt="" />;
};

const selectCardTheme = type => {
  switch (type) {
    case CARD_TYPE.TOP:
      return 'card-content--top';
    case CARD_TYPE.LEFT:
      return 'card-content--left';
    case CARD_TYPE.BOTTOM:
      return 'card-content--bottom';
    default:
      return 'card-content--top';
  }
};

const Card = ({ data, type }) => {
  const { id, logo, title, link, gaEvent = null, features, performance } = data;
  const isPionex = id === 'pionex';
  const isBinance = id === 'binance';
  const isAce = id === 'ace';
  const cardTheme = selectCardTheme(type);

  const renderFeatures = isLeftContent => {
    const featureTheme = isLeftContent ? 'card-left-features' : 'card-features';

    return (
      <div className={cx(featureTheme)}>
        {features.map(feature => (
          <div className={cx('card-feature')} key={feature.title}>
            <img src={feature.logo} alt="feature logo" />
            <div>
              <h3>{feature.title}</h3>
              <span
                className={cx({ 'pionex-theme': isPionex }, { 'binance-theme': isBinance }, { 'ace-theme': isAce })}
              >
                {feature.subTitle}
              </span>
              <span className={cx('card-feature--description')}>{feature.description}</span>
            </div>
          </div>
        ))}
      </div>
    );
  };

  const renderTable = () => (
    <table>
      <thead>
        <tr>{performance && performance.headers.map(header => <th key={header}>{header}</th>)}</tr>
      </thead>
      <tbody>
        {performance &&
          performance.rows.map(({ symbol, value, day, tag }, idx) => {
            return (
              <tr key={`row-${idx.toString()}`}>
                <td>
                  {renderSymbolIcon(symbol.toLowerCase())}
                  <span>{symbol}</span>
                  {tag && <span className={cx('symbol-tag')}>{tag}</span>}
                </td>
                <td>{value}</td>
                <td>{day}</td>
              </tr>
            );
          })}
      </tbody>
    </table>
  );

  const handleClick = () => {
    if (typeof window !== 'undefined') {
      window.parent.postMessage(gaEvent, '*');
      window.open(link, '_blank');
    }
  };

  const renderLeftContent = () => {
    return (
      <div>
        {renderFeatures(true)}
        {renderTable()}
      </div>
    );
  };

  const renderContent = () => {
    switch (type) {
      case CARD_TYPE.TOP:
        return renderFeatures();
      case CARD_TYPE.LEFT:
        return renderLeftContent();
      case CARD_TYPE.BOTTOM:
        return renderTable();
      default:
        return 'card-content--top';
    }
  };

  return (
    <div className={cx('card')} {...gaDataset(gaEvent)} onClick={handleClick}>
      <div className={cx('card-header')}>
        {renderCompanyLogo(logo)}
        <span className={cx('card-header--title')}>{title}</span>
      </div>
      <div className={cx('card-content', cardTheme)}>{renderContent()}</div>
    </div>
  );
};

Card.propTypes = {
  data: {
    id: PropTypes.oneOf('pionex', 'binance', 'ace', 'bincentive', 'pokket'),
    title: PropTypes.string,
    link: PropTypes.string,
    gaEvent: PropTypes.object,
    description: PropTypes.oneOfType(PropTypes.string, null),
    features: {
      title: PropTypes.string,
      subTitle: PropTypes.string,
      description: PropTypes.string,
    },
    performance: {
      headers: PropTypes.array,
      rows: PropTypes.arrayOf({
        symbol: PropTypes.string,
        value: PropTypes.string,
        day: PropTypes.string,
      }),
    },
  },
  type: PropTypes.oneOf([CARD_TYPE.TOP, CARD_TYPE.BOTTOM, CARD_TYPE.LEFT]),
};
Card.defaultProps = {
  data: {},
  type: 'small',
};

export default Card;
