import React from 'react';
import PropTypes from 'prop-types';
import idx from 'idx';
import classNames from 'classnames/bind';
import AnueApiFetch from '../../../utils/AnueApiFetch';
import AnueLink from '../../../utils/anueLink';
import gaDataset from '../../../utils/gaDataset';
import { arrayGroup, arrayShuffle } from '../../../utils/array';
import LoadingSkeleton from '../../LoadingSkeleton/LoadingSkeleton';
import { DOT_POSITION } from '../../Carousel/constants';
import Carousel from '../../Carousel/Class/Carousel';
import { LOGO_WALL_TYPE } from '../constants';
import styles from '../LogoWallCarousel.scss';

const cx = classNames.bind(styles);

export default class LogoWallCarousel extends React.PureComponent {
  static propTypes = {
    width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    height: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    delay: PropTypes.number,
    type: PropTypes.oneOf(Object.values(LOGO_WALL_TYPE)),
    dataPrefix: PropTypes.arrayOf(PropTypes.string),
    apiEnv: PropTypes.string,
  };

  static defaultProps = {
    width: 334,
    height: 230,
    delay: 10000,
    type: LOGO_WALL_TYPE.HOMEPAGE_D,
    dataPrefix: ['data-global-ga'],
    apiEnv: 'prod',
  };

  constructor(props) {
    super(props);
    this.state = {
      sources: [Array(4).fill(0), Array(4).fill(0), Array(4).fill(0)],
    };

    this.request = new AnueApiFetch();
    this.logoWallApiUrl = new AnueLink(props.apiEnv).logoWallApiUrl;
  }

  componentDidMount() {
    this.fetchLogoWall();
  }

  fetchLogoWall = async () => {
    const { type } = this.props;

    if (type === 'none') return;

    try {
      const rawResult = await this.request.get(`${this.logoWallApiUrl}${type}`);

      if (rawResult && rawResult.status === 200) {
        const sourceArr = idx(rawResult, _ => _.data.items);

        if (sourceArr) {
          arrayShuffle(sourceArr);
          let i = 0;

          while (sourceArr.length < 12) {
            sourceArr.push(sourceArr[i]);
            i += 1;
          }

          const newArray = arrayGroup(sourceArr, 4);

          this.setState({
            sources: newArray,
          });
        }
      }
    } catch (err) {
      console.error('[ERROR] Fetch logo wall have error!\n', err);
    }
  };

  renderLoadingSkeleton = index => (
    <div className={cx('slide__loading')} key={`logo_wall_item_loading_${index + 1}`}>
      <div className={cx('slide__loading--left')}>
        <div className={cx('slide__loading--left-image')}>
          <LoadingSkeleton />
        </div>
      </div>
      <div className={cx('slide__loading--right')}>
        <div className={cx('slide__loading--right-title')}>
          <LoadingSkeleton />
        </div>
        <div className={cx('slide__loading--right-subtitle')}>
          <LoadingSkeleton />
        </div>
      </div>
    </div>
  );

  renderItem = items => {
    const { dataPrefix } = this.props;

    return items.map((setting, index) => {
      return setting ? (
        <a
          key={`logo_wall_item_${index + 1}`}
          className={cx('slide__item')}
          href={setting.url}
          target="_blank"
          rel="noopener noreferrer"
          {...gaDataset({ dataPrefix, category: 'LogoWall', action: 'click', label: setting.title })}
          title={setting.title}
        >
          <div className={cx('slide__item--left')}>
            <img src={setting.image && setting.image.s.src} alt={setting.title} />
          </div>
          <div className={cx('slide__item--right')}>
            <h5>{setting.title}</h5>
            <p>{setting.subtitle}</p>
          </div>
        </a>
      ) : (
        this.renderLoadingSkeleton(index)
      );
    });
  };

  render() {
    const { width, height, delay } = this.props;
    const { sources } = this.state;
    const containerWidth = width.toString().includes('%') ? width : `${width}px`;
    let containerHeight = height;

    if (!height.toString().includes('%')) {
      const magicHeight = height + 12 + 7 + 2;

      containerHeight = `${magicHeight}px`;
    }

    return (
      <div style={{ width: containerWidth, height: containerHeight }}>
        <div className={cx('slide')}>
          <Carousel width={width} height={height} speed={delay} dotPosition={DOT_POSITION.FIXED}>
            {sources &&
              sources.map((items, index) => (
                <div key={`logo_wall_${index + 1}`} className={cx('slide--wrapper')}>
                  {this.renderItem(items)}
                </div>
              ))}
          </Carousel>
        </div>
      </div>
    );
  }
}
