import React, { useEffect, useState } from 'react';
import PropTypes from 'prop-types';
import idx from 'idx';
import classNames from 'classnames/bind';
import AnueApiFetch from '../../../utils/AnueApiFetch';
import AnueLink from '../../../utils/anueLink';
import gaDataset from '../../../utils/gaDataset';
import { arrayGroup, arrayShuffle } from '../../../utils/array';
import LoadingSkeleton from '../../LoadingSkeleton/LoadingSkeleton';
import { DOT_POSITION } from '../../Carousel/constants';
import Carousel from '../../Carousel/Hook/Carousel';
import { LOGO_WALL_TYPE } from '../constants';
import styles from '../LogoWallCarousel.scss';

const cx = classNames.bind(styles);

const LogoWallCarousel = ({ width, height, delay, type, dataPrefix, apiEnv }) => {
  const [sources, setSources] = useState([Array(4).fill(0), Array(4).fill(0), Array(4).fill(0)]);
  const logoWallApiUrl = new AnueLink(apiEnv).logoWallApiUrl;
  const request = new AnueApiFetch();

  const renderLoadingSkeleton = index => (
    <div className={cx('slide__loading')} key={`logo_wall_item_loading_${index + 1}`}>
      <div className={cx('slide__loading--left')}>
        <div className={cx('slide__loading--left-image')}>
          <LoadingSkeleton />
        </div>
      </div>
      <div className={cx('slide__loading--right')}>
        <div className={cx('slide__loading--right-title')}>
          <LoadingSkeleton />
        </div>
        <div className={cx('slide__loading--right-subtitle')}>
          <LoadingSkeleton />
        </div>
      </div>
    </div>
  );

  const renderItem = items => {
    return items.map((setting, index) => {
      return setting ? (
        <a
          key={`logo_wall_item_${index + 1}`}
          className={cx('slide__item')}
          href={setting.url}
          target="_blank"
          rel="noopener noreferrer"
          {...gaDataset({ dataPrefix, category: 'LogoWall', action: 'click', label: setting.title })}
          title={setting.title}
        >
          <div className={cx('slide__item--left')}>
            <img src={setting.image && setting.image.s.src} alt={setting.name} />
          </div>
          <div className={cx('slide__item--right')}>
            <h5>{setting.title}</h5>
            <p>{setting.subtitle}</p>
          </div>
        </a>
      ) : (
        renderLoadingSkeleton(index)
      );
    });
  };

  const containerWidth = width.toString().includes('%') ? width : `${width}px`;
  let containerHeight = height;

  if (!height.toString().includes('%')) {
    const magicHeight = height + 12 + 7 + 2;

    containerHeight = `${magicHeight}px`;
  }

  useEffect(
    () => {
      const fetchLogoWall = async () => {
        try {
          const rawResult = await request.get(`${logoWallApiUrl}${type}`);

          if (rawResult && rawResult.status === 200) {
            const sourceArr = idx(rawResult, _ => _.data.items);

            if (sourceArr) {
              arrayShuffle(sourceArr);
              let i = 0;

              while (sourceArr.length < 12) {
                sourceArr.push(sourceArr[i]);
                i += 1;
              }

              const newArray = arrayGroup(sourceArr, 4);

              setSources(newArray);
            }
          }
        } catch (err) {
          console.error('[ERROR] Fetch logo wall have error!\n', err);
        }
      };

      if (type === 'none') return;

      fetchLogoWall();
    },
    [type]
  );

  return (
    <div style={{ width: containerWidth, height: `${containerHeight}px` }}>
      <div className={cx('slide')}>
        <Carousel width={width} height={height} speed={delay} dotPosition={DOT_POSITION.FIXED}>
          {sources &&
            sources.map((items, index) => (
              <div key={`slide_${index + 1}`} className={cx('slide--wrapper')}>
                {renderItem(items)}
              </div>
            ))}
        </Carousel>
      </div>
    </div>
  );
};

LogoWallCarousel.propTypes = {
  width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  height: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  delay: PropTypes.number,
  type: PropTypes.oneOf(Object.values(LOGO_WALL_TYPE)),
  dataPrefix: PropTypes.arrayOf(PropTypes.string),
  apiEnv: PropTypes.string,
};

LogoWallCarousel.defaultProps = {
  width: 334,
  height: 230,
  delay: 10000,
  type: LOGO_WALL_TYPE.HOMEPAGE_D,
  dataPrefix: ['data-global-ga'],
  apiEnv: 'prod',
};

export default LogoWallCarousel;
